`default_nettype	none
`timescale			1ns/1ps
/* -----------------------------------------------------------------
	CX-Card4 System Controller		TAB = 4;
	Verilog-HDL Version

	Synthesis & Fitter tool = "QuartusII13.0"

	FileName = GPIF_QSYS_Verilog.v
	
	M.YOSHIDA				16.SEP.2010		REV	0.0
	PSI						28.MAY.2013    REV	1.0
----------------------------------------------------------------- */
//	Define Declaration
//	Module Declaration
module	QSYS_Verilog		(	ifclk1,				//	System clock 48MHz input
								rstn,				//	Active low reset input

								fd,					//	GPIF Bidirectional data bus
								rdn,
								wrn,
								cmdn,
								rgdtn,
								rdy,
								linkn,
								crc_in,
								
								sram_ab,
								sram_db,
								sram_csn,
								sram_wen,
								sram_oen,

								led,

								dipsw,				//	DIPSW input
								psw,
							
								univ2_seg_db,		// Card-UNIV2
								univ2_seg_sel		// Card-UNIV2								

						);

	//	Parameter  Declaration
	`include	"..\Param.v"
	
	//	Port declaration
	//	System Port
	input							ifclk1;
	input							rstn;
	
	//	GPIF Port
	inout	[15:0]					fd;
	input							rdn;
	input							wrn;
	input							cmdn;
	input							rgdtn;
	output							rdy;
	input							linkn;
	input							crc_in;

	//	Async-SRAM Port
	output	[MEM_LENGTH-1:0]		sram_ab;
	inout		[7:0]					sram_db;
	output							sram_csn;
	output							sram_wen;
	output							sram_oen;
	
	//	Switch Port
	input	[4:1]					dipsw;
	input							psw;
	
	//	LED & 7SEG-LED
	output	[3:0]					led;
	
	// Card-UNIV2
	output	[4:0]					univ2_seg_sel;
	output	[7:0]					univ2_seg_db;

	
	
	wire							pll_clk;
	wire							pll_locked;
	wire	[7:0]					led_wire;
	wire	[5:0]					rdy_wire;
	wire							bridge_wire;
	

	wire	[31:0]				fifo_rd;
	wire  [14:0]				fifo_num;	
	wire							fifo_wck; //2.4MHz
	wire							fifo_wfull;
	wire							cs;
	wire							fifo_wr;
	wire							read_req;
	wire							fifo_empty;

	
	Pll					pll1	(	.inclk0( ifclk1 ),
									.areset( !rstn ),
									.c0( pll_clk ),
									.c1( fifo_wck),
									.locked( pll_locked )
								);

    QSYS_top			qsys1	(	.clk_0_clk( pll_clk ),
									.reset_0_reset_n( !( !rstn | !pll_locked )),
									.gpif_master_0_conduit_end_fd( fd ),
									.gpif_master_0_conduit_end_ctl({ 1'b0, 1'b0, rgdtn, cmdn, wrn, rdn }),
									.gpif_master_0_conduit_end_rdy( rdy_wire ),

									.sram_conduit_end_ab( sram_ab ),
									.sram_conduit_end_db( sram_db ),
									.sram_conduit_end_csn( sram_csn ),
									.sram_conduit_end_wen( sram_wen ),
									.sram_conduit_end_oen( sram_oen ),
		
									.notuse_external_connection_export(),
									.led_external_connection_export( led_wire ),
									.swin_external_connection_export( ~{ 2'b11,fifo_wfull, psw, dipsw }),
									.pio_0_external_connection_export(),

									
									.generic_tristate_controller_0_tcm_read_out          (read_req),          // generic_tristate_controller_0_tcm.read_out
									.generic_tristate_controller_0_tcm_begintransfer_out (), //                                  .begintransfer_out
									.generic_tristate_controller_0_tcm_chipselect_out    (cs),    //                                  .chipselect_out
									.generic_tristate_controller_0_tcm_outputenable_out  (),  //                                  .outputenable_out
									.generic_tristate_controller_0_tcm_waitrequest_in    (fifo_empty & cs & read_req),    //                                  .waitrequest_in
									.generic_tristate_controller_0_tcm_request           (bridge_wire),           //                                  .request
									.generic_tristate_controller_0_tcm_grant             (bridge_wire),             //                                  .grant
									.generic_tristate_controller_0_tcm_data_in           (fifo_rd)            //                                  .data_in
								);
								
	dpfifo	dpfifo_inst (
									.aclr ( !rstn ),
									.data ( debug_cnt ), 
									.rdclk ( pll_clk), 	//Avalon
									.rdreq ( read_req & cs),//Avalon	
									.wrclk (fifo_wck  ), // counter
									.wrreq ( fifo_wr), //counter
									.q ( fifo_rd ),	// Avalon
									.rdempty ( fifo_empty ),
									.wrfull ( fifo_wfull ),
									.wrusedw ( fifo_num )
	);
	
	
/*--------  Debug counter --------*/
			reg	[7:0]				debug_cnt;
		always @(posedge fifo_wck or negedge rstn)
			begin
				if (~rstn)
					begin	
						debug_cnt <=8'h00;
					end
				else if (!fifo_wfull )
					begin
						debug_cnt <= debug_cnt + 1;
					end
				else 
					begin	
						debug_cnt <= debug_cnt;
					end
			end
/*----------------------------------*/

	
				
	assign	rdy		=	rdy_wire[0];

	assign 	fifo_wr = !fifo_wfull;
	

	assign	led		=	~( {fifo_empty,(read_req & cs),rdy_wire[0],fifo_wr} );
	
	
		DispCont		univ2	(	.clk( pll_clk ),
								.rstn( rstn ),
								.dis( 1'b0 ),
								.seg_in( {2'b00,fifo_num} ),
								.led_in( { debug_cnt }),
								.matrix_db( univ2_seg_db ),
								.matrix_sel( univ2_seg_sel )
					);
	
endmodule
